--
-- Sonic Adventure-esque monitor display
-- (c) Inuyasha 2014
-- Permission is granted to use this code in your own mods.
-- This code should be future-proof and work with all custom monitors.
--

local QUESTIONBOX_DETECT = 296396
local GRABICON = "GRABICO"	-- patch to use for the karma icon

local hu_roulette

local function draw_monitor(v, stplyr)
	if not stplyr.monitor_list or stplyr.monitor_time <= 0
		return
	end
	
	-- transparency handling
	local usertflag = v.userTransFlag()
	local trans_diff = ((V_90TRANS + V_10TRANS) - usertflag)>>V_ALPHASHIFT	-- shift to make it between 0-10

	local fade = usertflag
	local target = fade
	if stplyr.monitor_time < 20
		local t = (20 - stplyr.monitor_time)/2
		fade = max(target, V_10TRANS*t)
	end
	
	local roulette_frame
	
	for i, monitor in ipairs(stplyr.monitor_list)
		local draw1upIcon = false
		local sprite
		
		if monitor.grabbag	-- a
		and stplyr.mo and stplyr.mo.grabbag and stplyr.mo.grabbag.roulette
			
			local g = stplyr.mo.grabbag
			if g.roulette > TICRATE*6
				sprite = v.cachePatch(GRABICON)
			else
				local odds = MP_getMonitorOdds()
				local item = g.roulette and (g.roulette > TICRATE*2 and odds[N_RandomRange(1, #odds)][1]) or g.result
				local sstate = states[mobjinfo[mobjinfo[item].damage].spawnstate]
				if not hu_roulette or not (leveltime%2) and not (paused)
					
					if not roulette_frame
						hu_roulette = v.getSpritePatch(sstate.sprite, sstate.frame)
						roulette_frame = true
					end
					if not hu_roulette continue end
				end
				if g.roulette < TICRATE*2 and g.roulette%10 > 5 continue end
				sprite = hu_roulette
			end
		else
			sprite = v.cachePatch(monitor.spr)

			if (monitor.spr == "TV1UC0" and stplyr.mo and stplyr.mo.valid and P_IsValidSprite2(stplyr.mo, SPR2_LIFE))
				sprite = v.cachePatch("TV1PC0")
				draw1upIcon = true
			end
		end


		if not v.patchExists(monitor.spr)
			print("\x82".."WARNING: ".."\x80".. "Monitor type is out of range! (>>#".."\x82"..(monitor.info or "UNDENIFIED, PLEASE ASK A PROGRAMMER.").."\x80"..")")
			continue
		end
		
		v.drawScaled(monitor.x<<FRACBITS, 184<<FRACBITS, FRACUNIT, sprite, V_SNAPTOBOTTOM|fade)
		if (draw1upIcon) then v.drawScaled(monitor.x<<FRACBITS, 180<<FRACBITS, FRACUNIT, v.getSprite2Patch(stplyr.mo.skin, SPR2_LIFE, A), V_SNAPTOBOTTOM|fade, v.getColormap(TC_DEFAULT, stplyr.skincolor)) end
	end
end
hud.add(draw_monitor);

local function handle_monitors(player)

	if player.monitor_time == nil
		player.monitor_time = 0
	end

	if player.monitor_list and player.monitor_time > 0 and not player.spectator and player.mo and player.mo.health > 0
		if not (player.mo and player.mo.grabbag and player.mo.grabbag.roulette)
			player.monitor_time = $1 - 1
		end
	else
		player.monitor_time = nil
		player.monitor_list = nil
		return
	end

	for i, monitor in ipairs(player.monitor_list)

		local finalxpos = 160 + ((#player.monitor_list-1) * 16) - ((i-1) * 32)

		monitor.x = $1 + max(8, (finalxpos - monitor.x) / 6)

		if monitor.x > finalxpos
			monitor.x = finalxpos
		end
		
		if player.mo and player.mo.grabbag and player.mo.grabbag.roulette == 1
			monitor.grabbag = nil
		end
	end
end

local function hook_think()
	for player in players.iterate()
		handle_monitors(player)
	end
end
addHook("ThinkFrame", hook_think)
addHook("MapLoad", do
	for p in players.iterate do
		p.monitor_list = nil
		p.monitor_time = nil
	end
end)
