-- MonitorsPlus menus.......

local mp_menus = {
	"MPM_MAIN",
	"MPM_ODDS",
	"MPM_OPTS",
}

local mp_menuflags = {
	"MPF_SUBMENU",
	"MPF_MONITOR",
	"MPF_CVAR",
	"MPF_CVARNUM",
	"MPF_FUNC",
}

for i = 1, #mp_menus do
	rawset(_G, mp_menus[i], i)
end

for i = 1, #mp_menuflags do
	rawset(_G, mp_menuflags[i], i)
end

-- SRB2P library for input handling
addHook("MobjThinker", function(mo)
	if not mo.player return end
	local p = mo.player

	if not mo.P_inputs then mo.P_inputs = {} end	--setup input

	local buttons = {
	BT_JUMP, BT_USE, BT_ATTACK, BT_TOSSFLAG, BT_FIRENORMAL, BT_CAMRIGHT, BT_CAMLEFT,
	BT_CUSTOM1, BT_CUSTOM2, BT_CUSTOM3, BT_WEAPONPREV, BT_WEAPONNEXT }

	-- handle these normal buttons

	for i = 1, #buttons
		if p.cmd.buttons & buttons[i]
			if not mo.P_inputs[buttons[i]] then mo.P_inputs[buttons[i]] = 0 end
			mo.P_inputs[buttons[i]] = $+1
		else
			mo.P_inputs[buttons[i]] = 0
		end
	end

	-- handle weaponmask?
	-- get those with buttons 1 through 7

	for i = 1,7
		if p.cmd.buttons & BT_WEAPONMASK == i
			if not mo.P_inputs[i] then mo.P_inputs[i] = 0 end
			mo.P_inputs[i] = $+1
		else
			mo.P_inputs[i] = 0
		end
	end

	-- handle directions
	-- normal directional taps

	local directionfunc = {
		function(p) return p.cmd.forwardmove > 16 end,
		function(p) return p.cmd.forwardmove < -16 end,
		function(p) return p.cmd.sidemove > 16 end,
		function(p) return p.cmd.sidemove < -16 end,
	}

	local dirs = {"up", "down", "right", "left"}

	for k,v in ipairs(dirs)
		if directionfunc[k](p)
			if not mo.P_inputs[v] then mo.P_inputs[v] = 0 end
			mo.P_inputs[v] = $+1
		else
			mo.P_inputs[v] = 0
		end
	end
end, MT_PLAYER)

local function MP_exportConfig(p)
	if p == consoleplayer
		print("Exporting config, please stand by...")
		local file = io.openlocal("client/mp_config_export.txt", "w")
		
		-- cvars:
		local cvs = {
			cv_legacymonitors, 
			CV_FindVar("respawnitemtime"),
			cv_monitorweapons,
			cv_coopmonitors,
			cv_grabbag,
			cv_grabbagcd,
			cv_autograbbag
		}
		
		for i = 1, #cvs
			local str = cvs[i].name.." "..cvs[i].value
			file:write(str.."\n")
		end
		
		
		-- odds:
		local odds = MP_getMonitorOdds()
		for i = 1, #odds
			local str = "mp_setodds "..i.." "..odds[i][2].." "..odds[i][3].." "..odds[i][4]
			file:write(str.."\n")
		end
		
		file:close()
		
		print("Config succesfully exported in \'./luafiles/client/mp_config_export.txt\'")
		print("To use this config, move it to your root folder and use \'exec mp_config_export.txt\' on the title screen.")
	end
end

local mp_menustruct = {
	
	[MPM_MAIN] = {
		
		header = "MONITORSPLUS OPTIONS",
		center = true,
		x = 160,
		y = 80,
		
		choices = {
			{"Manage Monitors", MPF_SUBMENU, MPM_ODDS},
			{"Options", MPF_SUBMENU, MPM_OPTS},
		}
	},
	
	[MPM_OPTS] = {
		header = "TOGGLES AND VARIABLES",
		
		choices = {
			{"MONITOR OPTIONS"},
			{"VANILLA SPAWN METHOD", MPF_CVAR, cv_legacymonitors},
			{"MONITOR RESPAWN TIME", MPF_CVARNUM, CV_FindVar("respawnitemtime"), 1, 300},
			{"WEAPON RINGS FROM MONITORS", MPF_CVAR, cv_monitorweapons},
			{"SPAWN MONITORS IN COOP/SP", MPF_CVAR, cv_coopmonitors},
			{""},
			{"GRAB BAG OPTIONS"},
			{"ENABLE GRAB BAG", MPF_CVAR, cv_grabbag},
			{"GRAB BAG COOLDOWN", MPF_CVARNUM, cv_grabbagcd, 0, 999},
			{"AUTO USE GRAB BAG", MPF_CVAR, cv_autograbbag},
			{""},
			{"EXPORT CONFIG IN LUAFILES", MPF_FUNC, MP_exportConfig},			
		},
		
	},
}

local names = getMonitorSpawnList()

mp_menustruct[MPM_ODDS] = {choices = {}}
for i = 1, #names do
	mp_menustruct[MPM_ODDS].choices[i] = {names[i], MPF_MONITOR}
end

local function resetfunc(p)
	S_StartSound(nil, sfx_menu1, p)
	MP_resetOdds()
end
table.insert(mp_menustruct[MPM_ODDS].choices, {"RESET TO DEFAULTS", MPF_FUNC, resetfunc})

mp_menustruct[MPM_ODDS].header = "MANAGE MONITORS"
mp_menustruct[MPM_ODDS].scrollstart = 3
mp_menustruct[MPM_ODDS].y = 32

COM_AddCommand("mp_menu", function(p)
	
	if not p.mo or not p.mo.valid
		CONS_Printf(p, "This cannot be used while spectating")
		return
	end
	
	local mo = p.mo
	
	mo.MP_menu = {
		
		depth = MPM_MAIN,
		choice = 1,
		scroll = 0,
		
		monitor_selected = 0,
		monitor_choice = 1,
		player_choice = -1,
		
	}
end, COM_ADMIN)

rawset(_G, "MP_handleMenu", function(p)
	
	if p.mo and p.mo.valid and p.mo.MP_menu
		local mo = p.mo
		local m = mo.MP_menu
		
		local curr = mp_menustruct[m.depth].choices
		local scrollstart = mp_menustruct[m.depth].scrollstart or 9999
		
		-- force super/hyper to stay put.
		p.pflags = $|PF_FORCESTRAFE
		p.normalspeed = 0
		p.jumpfactor = 0
		p.charability2 = 0
		
		if not m.monitor_selected
			
			-- what's my first option?
			local firstopt = 1
			for i = 1, #curr
				if curr[i][2]
					firstopt = i
					break
				end
			end
			
			if mo.P_inputs["up"] == 1
			or mo.P_inputs["up"] > TICRATE/2
				m.choice = $-1
				
				while curr[m.choice] and not curr[m.choice][2]
					m.choice = $-1
				end
				
				if m.choice > scrollstart
				or m.scroll
					m.scroll = max(0, $-1)
				end
					
				if m.choice < firstopt
					m.choice = #curr
					m.scroll = max(0, m.choice - scrollstart)
				end
				S_StartSound(nil, sfx_menu1, p)
				
			elseif mo.P_inputs["down"] == 1
			or mo.P_inputs["down"] > TICRATE/2
				m.choice = $+1

				while curr[m.choice] and not curr[m.choice][2]
					m.choice = $+1
				end
				
				if m.choice > scrollstart
					m.scroll = $+1
				end
				
				if m.choice > #curr
					m.choice = firstopt
					m.scroll = 0
				end
				S_StartSound(nil, sfx_menu1, p)
			
			elseif mo.P_inputs[BT_JUMP] == 1
				local choice = curr[m.choice]
				if choice[2] == MPF_SUBMENU
					-- that's the only one we can use.
					
					m.depth = choice[3]					
					local newcurr = mp_menustruct[m.depth].choices
					
					local firstopt = 1
					for i = 1, #newcurr
						if newcurr[i][2]
							firstopt = i
							break
						end
					end
					
					m.choice = firstopt
					
					S_StartSound(nil, sfx_menu1, p)
				
				elseif choice[2] == MPF_MONITOR
					
					m.monitor_selected = m.choice
					S_StartSound(nil, sfx_menu1, p)
				
				elseif choice[2] == MPF_FUNC
					choice[3](p)
				end
				
			elseif mo.P_inputs[BT_SPIN] == 1
				
				if m.depth == MPM_MAIN
					mo.MP_menu = nil
					resetStats(p.mo)
					p.pflags = $ & ~PF_FORCESTRAFE
					return
				else
					
					m.choice = 1
					m.scroll = 0
					m.depth = MPM_MAIN
					S_StartSound(nil, sfx_menu1, p)
				end
			end
			
			local opt = curr[m.choice]
			
			-- dicey...
			if opt[2] == MPF_CVAR	-- CV_OnOff
				local cv = opt[3]
				
				if mo.P_inputs["left"] == 1
				or mo.P_inputs["left"] > TICRATE/2
				or mo.P_inputs["right"] == 1
				or mo.P_inputs["right"] > TICRATE/2
					COM_BufInsertText(p, cv.name.." "..(cv.value and "0" or "1"))
					S_StartSound(nil, sfx_menu1, p)
				end
			
			elseif opt[2] == MPF_CVARNUM
				
				local cv = opt[3]
				local mini = opt[4] or 0
				local maxi = opt[5] or 999
				
				if mo.P_inputs["right"] == 1
				or mo.P_inputs["right"] > TICRATE/2
					
					local newval = cv.value+1
					if newval > maxi
						newval = mini
					end
					
					COM_BufInsertText(p, cv.name.." "..(newval))
					S_StartSound(nil, sfx_menu1, p)					
				elseif mo.P_inputs["left"] == 1
				or mo.P_inputs["left"] > TICRATE/2

					local newval = cv.value-1
					if newval < mini
						newval = maxi
					end

					COM_BufInsertText(p, cv.name.." "..(newval))
					S_StartSound(nil, sfx_menu1, p)	
				end
			end
		else
			-- hack for monitor odds selection...
			
			local odds = MP_getMonitorOdds()
			local lookup = {4, 2, 3}	-- R, S, W
			
			local odd = odds[m.monitor_selected]
			
			if mo.P_inputs["up"] == 1
			or mo.P_inputs["up"] > TICRATE/2
				m.monitor_choice = $-1
				if not m.monitor_choice
					m.monitor_choice = 4
				end
				S_StartSound(nil, sfx_menu1, p)
			
			elseif mo.P_inputs["down"] == 1
			or mo.P_inputs["down"] > TICRATE/2
				m.monitor_choice = $+1
				if m.monitor_choice > 4
					m.monitor_choice = 1
				end
				S_StartSound(nil, sfx_menu1, p)				
			
			elseif mo.P_inputs["right"] == 1
			or mo.P_inputs["right"] > TICRATE/2
				
				if m.monitor_choice < 4
					
					odd[lookup[m.monitor_choice]] = $+1
					
					if odd[lookup[m.monitor_choice]] > 999
						odd[lookup[m.monitor_choice]] = 0
					end
				
				else	-- player selection
					m.player_choice = $+1
					if m.player_choice > #players-1
						m.player_choice = -1
					end
					S_StartSound(nil, sfx_menu1, p)	
				end
				
				S_StartSound(nil, sfx_menu1, p)	
			
			elseif mo.P_inputs["left"] == 1
			or mo.P_inputs["left"] > TICRATE/2
				
				if m.monitor_choice < 4
				
					odd[lookup[m.monitor_choice]] = $-1
					
					if odd[lookup[m.monitor_choice]] < 0
						odd[lookup[m.monitor_choice]] = 999
					end
				
				else	-- player selection
					m.player_choice = $-1
					if m.player_choice < -1
						m.player_choice = #players-1
					end
					S_StartSound(nil, sfx_menu1, p)						
				end
				
				S_StartSound(nil, sfx_menu1, p)					
			
			elseif mo.P_inputs[BT_JUMP] == 1
			or mo.P_inputs[BT_SPIN] == 1
				
				if mo.P_inputs[BT_JUMP] == 1
				and m.monitor_choice == 4
					if m.player_choice == -1 or (players[m.player_choice] and players[m.player_choice].valid)
						COM_BufInsertText(p, "mp_spawnmonitor "..m.player_choice.." "..m.monitor_selected)
						S_StartSound(nil, sfx_menu1, p)
					end
					return
				end
				
				m.monitor_selected = 0
				m.monitor_choice = 1
				S_StartSound(nil, sfx_menu1, p)
			end
		end
	end
end)

-- menu display
rawset(_G, "HU_handleMenu", function(v, p)
	if p.mo and p.mo.valid and p.mo.MP_menu
		v.fadeScreen(31, 7)
		
		local d = p.mo.MP_menu
		local m = mp_menustruct[p.mo.MP_menu.depth]
		
		v.drawString(160, 10, m.header or "", V_SNAPTOTOP|V_YELLOWMAP, "center")
		
		-- draw opts:
		if m.choices
			local y = m.y or 30
			local x = m.x or 20
			
			local centered = m.center and "center" or nil
			
			local menuchoices = m.choices
			
			for i = d.scroll +1, #menuchoices
				local c = menuchoices[i]
				if (c[2] == MPF_SUBMENU or c[2] == MPF_CVAR or c[2] == MPF_CVARNUM or c[2] == MPF_FUNC or not c[2])
				
					local selected = (i == d.choice)
					local prefix = selected and ">" or ""
					
					v.drawString(x, y, prefix..c[1], (not c[2] or i == d.choice) and V_YELLOWMAP or 0, centered)
					
					if c[2] == MPF_CVAR
					or c[2] == MPF_CVARNUM
						local cv = c[3]
						local str = ""
						if c[2] == MPF_CVAR
							str = cv.value and "ON" or "OFF"
						else
							str = cv.value
						end
						
						local arr_r = ""
						local arr_l = ""
						if selected
							arr_r = (leveltime%20 < 10) and " >" or ">"
							arr_l = (leveltime%20 < 10) and "< " or "<"
						end
						
						v.drawString(270, y, arr_l..str..arr_r, (not c[2] or i == d.choice) and V_YELLOWMAP or 0, "center")
					end
					
					y = $+10
				elseif c[2] and c[2] == MPF_MONITOR
					
					local dta = c[1]
					
					-- now we can have some fun
					-- get the monitor sprite
					local mt = dta.mobj
					local tvinfo = mobjinfo[mt]
					local info = mobjinfo[tvinfo.damage]	-- this gives us the icon 
					local state = states[info.spawnstate]
					local sprite = state.sprite
					local frame = state.frame
					local spritedraw = v.getSpritePatch(sprite, frame)
					local odds = MP_getMonitorOdds()[i]
					
					v.draw(x, y+8, spritedraw)
					
					if d.monitor_selected != i
					
						v.drawString(x+20, y-7, "#"..i.." "..dta.name.." MONITOR", i == d.choice and V_YELLOWMAP or 0)
						
						local ox = x+20
						local pref = {"RANDOM:", "STRONG:", "WEAK:"}
						local order = {4, 2, 3}
						
						for j = 2, #odds
							
							local str = "\x86"..pref[j-1].."\x80".." "
							str = $..odds[order[j-1]]
							
							v.drawString(ox, y+1, str)
							
							ox = $+ v.stringWidth(str) + 8
						end
					else
						
						local cy = y-14
						local order = {4, 2, 3}
						
						local pref = {"RANDOM ", "STRONG", "WEAK ", "SPAWN ON "}
						for j = 1, 4
							
							local selected = d.monitor_choice == j
							
							local arr_r = ""
							local arr_l = ""
							
							if selected
								arr_r = ">"
								arr_l = "<"
								if leveltime%20 < 10
									arr_r = " >"
									arr_l = "< "
								end
							end
							
							v.drawString(x+20, cy, pref[j], selected and V_YELLOWMAP or 0)
							
							if j < 4
								v.drawString(x+110, cy, arr_l..odds[order[j]]..arr_r, selected and V_YELLOWMAP or 0, "center")
							else
								local pname = ""
								if d.player_choice == -1
									pname = "All Players"
								else
									if players[d.player_choice] and players[d.player_choice].valid
										pname = players[d.player_choice].name
									else
										pname = "INVALID"
									end
								end
								
								v.drawString(x+160, cy, arr_l.."("..d.player_choice..") "..pname..arr_r, selected and V_YELLOWMAP or 0, "center")
							end
							cy = $+8
						end
					end
					
					y = $+32
				end
			end
		end
	end
end)